@@ -9,14 +9,16 @@ import os.path
 
 import seaborn as sns
 import matplotlib.pyplot as plt
+import matplotlib.colors as mcol
 import matplotlib.ticker as mticker
 import matplotlib.gridspec as gridspec
-from matplotlib.patches import PathPatch, Patch
+from matplotlib.patches import Patch, PathPatch
+from matplotlib.lines import Line2D
 from cartopy import config as cconfig
 import cartopy.feature as cfeature
 from cartopy.mpl.gridliner import LONGITUDE_FORMATTER, LATITUDE_FORMATTER
 from pygeogrids.grids import BasicGrid, genreg_grid
-from shapely.geometry import Polygon
+from shapely.geometry import Polygon, Point
 
 import warnings
 
@@ -126,7 +128,7 @@ def geotraj_to_geo2d(df, index=globals.index_names, grid_stepsize=None):
 
     return zz, data_extent, origin
 
-def get_value_range(ds, metric=None, force_quantile=False, quantiles=[0.025, 0.975], type=None):
+def get_value_range(ds, metric=None, force_quantile=False, quantiles=[0.025, 0.975], diff_map=False):
     """
     Get the value range (v_min, v_max) from globals._metric_value_ranges
     If the range is (None, None), a symmetric range around 0 is created,
@@ -145,6 +147,8 @@ def get_value_range(ds, metric=None, force_quantile=False, quantiles=[0.025, 0.9
     quantiles : list, optional
         quantile of data to include in the range.
         The default is [0.025,0.975]
+    diff_map : bool, default is False
+        Whether the colorbar is for a difference plot
 
     Returns
     -------
@@ -156,11 +160,7 @@ def get_value_range(ds, metric=None, force_quantile=False, quantiles=[0.025, 0.9
     if metric == None:
         force_quantile = True
 
-    if not type == None and type == 'diff':
-        ranges = globals._diff_value_ranges
-    else:
-        ranges = globals._metric_value_ranges
-
+    ranges = globals._metric_value_ranges
     if not force_quantile:  # try to get range from globals
         try:
             v_min = ranges[metric][0]
@@ -185,6 +185,10 @@ def get_value_range(ds, metric=None, force_quantile=False, quantiles=[0.025, 0.9
 
     if force_quantile:  # get quantile range
         v_min, v_max = get_quantiles(ds, quantiles)
+        # adjust range based on the difference values in the map
+        if diff_map:
+            extreme = max([abs(v) for v in get_quantiles(ds, quantiles)])
+            v_min, v_max = -extreme, extreme
 
     return v_min, v_max
 
@@ -233,8 +237,9 @@ def get_plot_extent(df, grid_stepsize=None, grid=False):
         (x_min, x_max, y_min, y_max) in Data coordinates.
     
     """
-    lat, lon = globals.index_names
+    lat, lon, gpi = globals.index_names
     if grid and grid_stepsize in ['nan', None]:
+        # todo: problem if only single lon/lat point is present?
         x_min, x_max, dx, len_x = _get_grid(df.index.get_level_values(lon))
         y_min, y_max, dy, len_y = _get_grid(df.index.get_level_values(lat))
         extent = [x_min-dx/2., x_max+dx/2., y_min-dx/2., y_max+dx/2.]
@@ -303,9 +308,13 @@ def get_extend_cbar(metric):
         else:
             return 'neither'
 
-def style_map(ax, plot_extent, add_grid=True, map_resolution=globals.naturalearth_resolution,
-              add_topo=False, add_coastline=True,
-              add_land=True, add_borders=True, add_us_states=False):
+def style_map(
+        ax, plot_extent, add_grid=True,
+        map_resolution=globals.naturalearth_resolution,
+        add_topo=False, add_coastline=True,
+        add_land=True, add_borders=True, add_us_states=False,
+        grid_intervals=globals.grid_intervals,
+):
     ax.set_extent(plot_extent, crs=globals.data_crs)
     ax.spines["geo"].set_linewidth(0.4)
     if add_grid:
@@ -314,9 +323,9 @@ def style_map(ax, plot_extent, add_grid=True, map_resolution=globals.naturaleart
         try:
             grid_interval = max((plot_extent[1] - plot_extent[0]),
                                 (plot_extent[3] - plot_extent[2])) / 5  # create apprx. 5 gridlines in the bigger dimension
-            if grid_interval <= min(globals.grid_intervals):
+            if grid_interval <= min(grid_intervals):
                 raise RuntimeError
-            grid_interval = min(globals.grid_intervals, key=lambda x: abs(
+            grid_interval = min(grid_intervals, key=lambda x: abs(
                 x - grid_interval))  # select the grid spacing from the list which fits best
             gl = ax.gridlines(crs=globals.data_crs, draw_labels=False,
                               linewidth=0.5, color='grey', linestyle='--',
@@ -338,7 +347,7 @@ def style_map(ax, plot_extent, add_grid=True, map_resolution=globals.naturaleart
                 gltext.xformatter = LONGITUDE_FORMATTER
                 gltext.yformatter = LATITUDE_FORMATTER
                 gltext.top_labels = False
-                gltext.left_labels = False
+                gltext.right_labels = False
                 gltext.xlocator = mticker.FixedLocator(xticks)
                 gltext.ylocator = mticker.FixedLocator(yticks)
             except RuntimeError as e:
@@ -401,11 +410,11 @@ def make_watermark(fig, placement=globals.watermark_pos, for_map=False, offset=0
                      xycoords='figure fraction', textcoords='offset points')
         bottom = fig.subplotpars.bottom
         if not for_map:
-            fig.subplots_adjust(bottom=bottom + offset)  # defaults to rc when none!
+            fig.subplots_adjust(bottom=bottom + offset)
     else:
         raise NotImplementedError
 
-def _make_cbar(fig, im, cax, ref_short:str, metric:str, label=None):
+def _make_cbar(fig, im, cax, ref_short:str, metric:str, label=None, diff_map=False):
     """
     Make colorbar to use in plots
 
@@ -423,6 +432,8 @@ def _make_cbar(fig, im, cax, ref_short:str, metric:str, label=None):
         name of metric
     label: str
         label to describe the colorbar
+    diff_map : bool, default is False
+        Whether the colorbar is for a difference plot
     """
     if label is None:
         try:
@@ -433,6 +444,8 @@ def _make_cbar(fig, im, cax, ref_short:str, metric:str, label=None):
             raise Exception('The metric \'{}\' or reference \'{}\' is not known.\n'.format(metric, ref_short) + str(e))
 
     extend = get_extend_cbar(metric)
+    if diff_map:
+        extend = "both"
     cbar = fig.colorbar(im, cax=cax, orientation='horizontal', extend=extend)
     cbar.set_label(label, weight='normal')
     cbar.outline.set_linewidth(0.4)
@@ -478,6 +491,17 @@ def _CI_difference(fig, ax, ci):
             horizontalalignment="center"
         )
 
+def _add_dummies(df:pd.DataFrame, to_add:int) -> list:
+    """
+    Add empty columns in dataframe to avoid error in matplotlib when not all boxplot groups have the same
+    number of values
+    """
+    for n, col in enumerate(np.arange(to_add)):
+        # add columns while avoiding name clashes
+        df[str(n)] = np.nan
+
+    return df
+
 def patch_styling(
         box_dict,
         facecolor
@@ -510,7 +534,7 @@ def boxplot(
     ----------
     df : pandas.DataFrame
         DataFrame containing 'lat', 'lon' and (multiple) 'var' Series.
-    ci: list
+    ci : list
         list of Dataframes containing "upper" and "lower" CIs
     label : str, optional
         Label of the y axis, describing the metric. The default is None.
@@ -518,7 +542,7 @@ def boxplot(
         Figure size in inches. The default is globals.map_figsize.
     dpi : int, optional
         Resolution for raster graphic output. The default is globals.dpi.
-    spacing: float, optional.
+    spacing : float, optional.
         Space between the central boxplot and the CIs. Default is 0.3
 
     Returns
@@ -542,8 +566,14 @@ def boxplot(
         for n, intervals in enumerate(ci):
             lower.append(intervals["lower"])
             upper.append(intervals["upper"])
-        lower = pd.concat(lower, ignore_index=True, axis=1)
-        upper = pd.concat(upper, ignore_index=True, axis=1)
+        lower = _add_dummies(
+            pd.concat(lower, ignore_index=True, axis=1),
+            len(center_pos)-len(ci),
+        )
+        upper = _add_dummies(
+            pd.concat(upper, ignore_index=True, axis=1),
+            len(center_pos)-len(ci),
+        )
         low = lower.boxplot(
             positions=center_pos - spacing,
             showfliers=False,
@@ -596,7 +626,7 @@ def mapplot(
         label=None,
         figsize=globals.map_figsize,
         dpi=globals.dpi,
-        diff_range=None,
+        diff_map=False,
         **style_kwargs
 ):
         """
@@ -607,32 +637,31 @@ def mapplot(
         ----------
         df : pandas.Series
             values to be plotted. Generally from metric_df[Var]
-        metric: str
+        metric : str
             name of the metric for the plot
-        ref_short: str
+        ref_short : str
                 short_name of the reference dataset (read from netCDF file)
-        ref_grid_stepsize: float or None, optional (None by default)
+        ref_grid_stepsize : float or None, optional (None by default)
                 angular grid stepsize, needed only when ref_is_angular == False,
-        plot_extent: tuple
+        plot_extent : tuple
                 (x_min, x_max, y_min, y_max) in Data coordinates. The default is None.
-        colormap:  Colormap, optional
+        colormap :  Colormap, optional
                 colormap to be used.
                 If None, defaults to globals._colormaps.
-        projection:  cartopy.crs, optional
+        projection :  cartopy.crs, optional
                 Projection to be used. If none, defaults to globals.map_projection.
                 The default is None.
-        add_cbar: bool, optional
+        add_cbar : bool, optional
                 Add a colorbar. The default is True.
         label : str, optional
             Label of the y axis, describing the metric. If None, a label is autogenerated from metadata.
             The default is None.
-        figsize: tuple, optional
+        figsize : tuple, optional
             Figure size in inches. The default is globals.map_figsize.
-        dpi: int, optional
+        dpi : int, optional
             Resolution for raster graphic output. The default is globals.dpi.
-        diff_range: None, 'adjusted' or 'fixed'
-            if none, globals._metric_value_ranges is used to define the bar extent; if 'fixed', globals._diff_value_ranges
-            is used instead; if 'adjusted', max and minimum values are used
+        diff_map : bool, default is False
+            if True, a difference colormap is created
         **style_kwargs :
             Keyword arguments for plotter.style_map().
 
@@ -642,18 +671,18 @@ def mapplot(
             the boxplot
         ax : matplotlib.axes.Axes
         """
-        v_min, v_max = get_value_range(df, metric)  # range of values
-        if diff_range and diff_range == 'adjusted':
-            v_min, v_max = get_value_range(df, metric=None)
-        elif diff_range and diff_range == 'fixed':
-            v_min, v_max = get_value_range(df, metric, type='diff')
-
-        # initialize plot
-        fig, ax, cax = init_plot(figsize, dpi, add_cbar, projection)
         if not colormap:
             cmap = globals._colormaps[metric]
         else:
             cmap = colormap
+        v_min, v_max = get_value_range(df, metric)
+        # everything changes if the plot is a difference map
+        if diff_map:
+            v_min, v_max = get_value_range(df, metric=None, diff_map=True)
+            cmap = globals._diff_colormaps[metric]
+
+        # initialize plot
+        fig, ax, cax = init_plot(figsize, dpi, add_cbar, projection)
 
         # scatter point or mapplot
         if ref_short in globals.scattered_datasets:  # scatter
@@ -661,7 +690,7 @@ def mapplot(
                 plot_extent = get_plot_extent(df)
 
             markersize = globals.markersize ** 2
-            lat, lon = globals.index_names
+            lat, lon, gpi = globals.index_names
             im = ax.scatter(df.index.get_level_values(lon),
                             df.index.get_level_values(lat),
                             c=df, cmap=cmap, s=markersize,
@@ -671,6 +700,8 @@ def mapplot(
         else:  # mapplot
             if not plot_extent:
                 plot_extent = get_plot_extent(df, grid_stepsize=ref_grid_stepsize, grid=True)
+            if isinstance(ref_grid_stepsize, np.ndarray):
+                ref_grid_stepsize = ref_grid_stepsize[0]
             zz, zz_extent, origin = geotraj_to_geo2d(df, grid_stepsize=ref_grid_stepsize)  # prep values
             im = ax.imshow(zz, cmap=cmap, vmin=v_min,
                            vmax=v_max, interpolation='nearest',
@@ -678,81 +709,134 @@ def mapplot(
                            transform=globals.data_crs, zorder=2)
 
         if add_cbar:  # colorbar
-            _make_cbar(fig, im, cax, ref_short, metric, label=label)
+            _make_cbar(fig, im, cax, ref_short, metric, label=label, diff_map=diff_map)
         style_map(ax, plot_extent, **style_kwargs)
         fig.canvas.draw()  # very slow. necessary bcs of a bug in cartopy: https://github.com/SciTools/cartopy/issues/1207
 
         return fig, ax
 
-def diff_plot(df:pd.DataFrame, **kwargs):
+def plot_spatial_extent(
+        polys:dict,
+        ref_points:bool=None,
+        overlapping:bool=False,
+        intersection_extent:tuple=None,
+        reg_grid=False,
+        grid_stepsize=None,
+        **kwargs,
+):
     """
-    Create a Bland Altman plot for a Dataframe and a list of other Dataframes. Difference is other - reference.
-
-    Parameters
-    ----------
-    ref_df : pd.DataFrame
-        Dataframe of the reference values
-
-    Returns
-    -------
-    fig : matplotlib.figure.Figure
-        the boxplot
-    ax : matplotlib.axes.Axes
-    """
-    fig, ax = plt.subplots(figsize=(16,10))
-
-    mean = "Mean with {}".format(df.columns[0])
-    diff = "Difference with {}".format(df.columns[0])
-
-    df[diff] = df.iloc[:,1] - df.iloc[:,0]
-    df[mean] = np.mean(df, axis=1)
-    md = np.mean(df[diff])
-    sd = np.std(df[diff])
-    ax = sns.scatterplot(x=df[mean], y=df[diff], **kwargs)
-    # mean line
-    ax.axhline(md, linestyle='-', label="Mean difference with {}".format(df.columns[1]))
-    # higher STD bound
-    ax.axhline(md + 1.96*sd, linestyle='--', label="Standard intervals")
-    # lower STD bound
-    ax.axhline(md - 1.96*sd, linestyle='--')
-
-    plt.legend()
-
-    return fig, ax
-
-def plot_spatial_extent(polys:dict, output:str=None, title:str=None):
-    """
-    Plots the given Polygons on a map.
+    Plots the given Polygons and optionally the reference points on a map.
 
     Parameters
     ----------
     polys : dict
         dictionary with shape {name: shapely.geometry.Polygon}
-    title : str
-        plot title
+    ref_points : 2D array
+        array of lon, lat for the reference points positions
+    overlapping : bool, dafault is False.
+        Whether the polygons have an overlap
+    intersection_extent : tuple | None
+        if given, corresponds to the extent of the intersection. Shape (minlon, maxlon, minlat, maxlat)
+    reg_grid : bool, default is False,
+        plotting oprion for regular grids (satellites)
     """
+    # todo: adjust figsize to shown polygons
     fig, ax, cax = init_plot(figsize=globals.map_figsize, dpi=globals.dpi)
+    legend_elements = []
+    # plot polygons
     for n, items in enumerate(polys.items()):
         name, Pol = items
         if n == 0:
             union = Pol
-        union = union.union(Pol)  # get maximum extent
-        try:
+         # get maximum extent
+        union = union.union(Pol)
+        style = {'color':'powderblue', 'alpha':0.4}
+        # shade the union/intersection of the polygons
+        if overlapping:
             x, y = Pol.exterior.xy
-            if name == output:
-                style = {'color':'powderblue', 'alpha':0.4}
-                ax.fill(x, y, label=name, **style, zorder=5)
+            if name == "selection":
+                ax.fill(x, y, **style, zorder=5)
                 continue
             ax.plot(x, y, label=name)
-        except:
-            pass
-
-    plt.legend(loc='upper right')
-    ax.set_title(title)
+        # shade the areas individually
+        else:
+            if name == "selection":
+                continue
+            x, y = Pol.exterior.xy
+            ax.fill(x, y, **style, zorder=6)
+            ax.plot(x, y, label=name, zorder=6)
+    # add reference points to the figure
+    if ref_points is not None:
+        if overlapping and intersection_extent is not None:
+            minlon, maxlon, minlat, maxlat = intersection_extent
+            mask = (ref_points[:,0]>=minlon) & (ref_points[:,0]<=maxlon) &\
+                   (ref_points[:,1]>=minlat) & (ref_points[:,1]<=maxlat)
+            selected = ref_points[mask]
+            outside = ref_points[~ mask]
+        else:
+            selected, outside = ref_points, np.array([])
+        marker_styles = [
+            {"marker": "o", "c":"turquoise", "s":15},
+            {"marker": "o", "c":"tomato", "s":15},
+        ]
+        # mapplot with imshow for gridded (non-ISMN) references
+        if reg_grid:
+            plot_df = []
+            for n, (point_set, style, name) in enumerate(zip(
+                    (selected, outside),
+                    marker_styles,
+                    ("Selected reference validation points", "Validation points outside selection")
+            )):
+                if point_set.size != 0:
+                    point_set = point_set.transpose()
+                    index = pd.MultiIndex.from_arrays(point_set, names=('lon', 'lat'))
+                    point_set = pd.Series(
+                        data=n,
+                        index=index,
+                    )
+                    plot_df.append(point_set)
+                    # plot point to 'fake' legend entry
+                    ax.scatter(0, 0, label=name, marker="s", s=10, c=style["c"])
+                else:
+                    continue
+            plot_df = pd.concat(plot_df, axis=0)
+            zz, zz_extent, origin = geotraj_to_geo2d(
+                plot_df,
+                grid_stepsize=grid_stepsize
+            )
+            cmap = mcol.LinearSegmentedColormap.from_list('mycmap', ['turquoise', 'tomato'])
+            im = ax.imshow(
+                zz, cmap=cmap,
+                origin=origin, extent=zz_extent,
+                transform=globals.data_crs, zorder=4
+            )
+        # scatterplot for ISMN reference
+        else:
+            for point_set, style, name in zip(
+                    (selected, outside),
+                    marker_styles,
+                    ("Selected reference validation points", "Validation points outside selection")
+            ):
+                if point_set.size != 0:
+                    im = ax.scatter(
+                        point_set[:,0], point_set[:,1],
+                        edgecolors='black', linewidths=0.1,
+                        zorder=4, transform=globals.data_crs,
+                        **style, label=name
+                    )
+                else:
+                    continue
+    # create legend
+    plt.legend(bbox_to_anchor=(1, 1), fontsize='medium')
+    # style plot
+    make_watermark(fig, globals.watermark_pos, offset=0)
+    title_style = {"fontsize": 12}
+    ax.set_title("Spatial extent of the comparison", **title_style)
     # provide extent of plot
     d_lon = abs(union.bounds[0] - union.bounds[2])* 1/8
     d_lat = abs(union.bounds[1] - union.bounds[3])* 1/8
     plot_extent = (union.bounds[0] - d_lon, union.bounds[2] + d_lon,
                    union.bounds[1] - d_lat, union.bounds[3] + d_lat)
-
-    style_map(ax, plot_extent)
+    plt.tight_layout()
+    grid_intervals = [1, 5, 10, 30]
+    style_map(ax, plot_extent, grid_intervals=grid_intervals)
